# Color Codes
BOLD = '\033[1m'
YELLOW = '\033[33m'
RED = '\033[31m'
RESET = '\033[0m'

# Prompt the user with the ai's generated command and makes sure they want to run it. 
class SafetyChecker:
    @staticmethod
    def confirm_and_run(command: str) -> bool:
        print(f"{BOLD}{YELLOW}\nGenerated command:{RESET}", command)
        confirm = input(f"{BOLD}Press Enter to run, or type 'n' to cancel: {RESET}").strip().lower()
        if confirm and confirm != 'y':
            print(f"{RED}Command cancelled.{RESET}")
            return False
        return True



import subprocess
import sys

class CommandRunner:
    @staticmethod
    def run(command: str):
        try:
            result = subprocess.run(command, shell=True, capture_output=True, text=True)
            if result.stdout:
                print(result.stdout, end='')
            if result.stderr:
                print(result.stderr, end='', file=sys.stderr)
            if result.returncode != 0:
                print(f"[Command exited with code {result.returncode}]", file=sys.stderr)
        except Exception as e:
            print(f"[Error running command: {e}]", file=sys.stderr)



import pytest
from aegish.commandrunner import CommandRunner
import io
import sys

def test_commandrunner_echo():
    runner = CommandRunner()
    captured_out = io.StringIO()
    sys_stdout = sys.stdout
    sys.stdout = captured_out
    runner.run('echo Hello World')
    sys.stdout = sys_stdout
    output = captured_out.getvalue()
    assert 'Hello World' in output




import subprocess
import shutil

# it handles ollama... genius naming right?

class OllamaHandler:
    def __init__(self):
        from aegish.config import load_config
        config = load_config()
        self.model = config["model"]["api"]
        self.exe = shutil.which("ollama")
        if not self.exe:
            raise FileNotFoundError("ollama CLI not found")

    def generate(self, prompt: str) -> str:
        candidates = [
            [self.exe, "run", self.model, prompt],
            [self.exe, "chat", self.model, prompt],
            [self.exe, "predict", self.model, prompt],
            [self.exe, "generate", self.model, prompt],
            [self.exe, "completion", self.model, prompt],
            [self.exe, "infer", self.model, prompt],
        ]
        errors = []
        for cmd in candidates:
            try:
                completed = subprocess.run(cmd, capture_output=True, text=True, timeout=20)
            except FileNotFoundError:
                raise FileNotFoundError("ollama CLI not found")
            except Exception as e:
                errors.append(f"{cmd}: {e}")
                continue
            out = (completed.stdout or completed.stderr or '').strip()
            if completed.returncode == 0 and out:
                return out
            errors.append(f"{cmd} returned {completed.returncode}: {out}")
        raise RuntimeError("No working ollama CLI command found. Attempts:\n" + "\n".join(errors))



import anthropic
from aegish.config import load_config


class ClaudeHandler:
    def __init__(self):
        config = load_config()
        self.model = config["model"]["api"]
        self.api_key = config.get('api_key')
        if not self.api_key:
            raise ValueError("No Claude API key found in config.")
        self.client = anthropic.Anthropic(api_key=self.api_key)

    def generate(self, prompt: str) -> str:
        message = self.client.messages.create(
            model=self.model,
            max_tokens=1000,
            messages=[
                {"role": "user", "content": prompt}
            ]
        )
        return message.content if hasattr(message, 'content') else str(message)



from google import genai
from aegish.config import load_config
from aegish.debug import print_debug


class GeminiHandler:
    def __init__(self):
        config = load_config()
        api_key = config.get('api_key')
        model = config.get('model').get('api')
        if not api_key:
            raise ValueError("No Gemini API key found in config.")
        self.model = model
        self.client = genai.Client(api_key=api_key)
        print_debug(f"GeminiHandler model: {self.model}")
        print_debug(f"GeminiHandler client: {self.client}")

    def generate(self, prompt: str) -> str:
        response = self.client.models.generate_content(
            model=self.model, contents=prompt
        )
        print_debug(f"Gemini API raw response: {response}")
        return response.text




import re

class PostProcessor:
    @staticmethod
    def clean(command: str) -> str:
        # Remove triple backticks and language tags
        # Even though the system prompt says only the command,
        # LLMs still add tags so this strips those and just returns the raw command
        command = re.sub(r'^```[a-zA-Z]*\s*', '', command.strip())
        command = re.sub(r'```$', '', command.strip())
        command = command.strip('`').strip()
        command = re.sub(r'^(shell|bash|sh|zsh)\s*', '', command, flags=re.IGNORECASE)
        return command



# A list of all the providers, models, and there api code names that aegish supports   
# Used in config.py
PROVIDERS = {
    "gemini": {
        "models": [
            {"api": "gemini-2.5-pro", "name": "Gemini 2.5 Pro"},
            {"api": "gemini-2.5-flash", "name": "Gemini 2.5 Flash"},
            {"api": "gemini-2.5-flash-lite", "name": "Gemini 2.5 Flash Lite"},
            {"api": "gemini-2.0-flash", "name": "Gemini 2.0 Flash"},
            {"api": "gemini-2.0-flash-lite", "name": "Gemini 2.0 Flash Lite"},
        ]
    },
    "claude": {
        "models": [
            {"api": "claude-opus-4-1-20250805", "name": "Claude Opus 4.1"},
            {"api": "claude-opus-4-20250514", "name": "Claude Opus 4"},
            {"api": "claude-sonnet-4-20250514", "name": "Claude Sonnet 4"},
            {"api": "claude-3-7-sonnet-20250219", "name": "Claude 3.7 Sonnet"},
            {
                "api": "claude-3-7-sonnet-latest",
                "name": "Claude 3.7 Sonnet (latest alias)",
            },
            {"api": "claude-3-5-haiku-20241022", "name": "Claude 3.5 Haiku"},
            {
                "api": "claude-3-5-haiku-latest",
                "name": "Claude 3.5 Haiku (latest alias)",
            },
            {"api": "claude-3-haiku-20240307", "name": "Claude 3 Haiku"},
        ]
    },
    "openai": {
        "api": "openai",
        "models": [
            {"api": "gpt-5-2025-08-07", "name": "GPT-5"},
            {"api": "gpt-5-mini-2025-08-07", "name": "GPT-5 Mini"},
            {"api": "gpt-5-nano-2025-08-07", "name": "GPT-5 Nano"},
            {"api": "gpt-4.1-2025-04-14", "name": "GPT-4.1"},
        ],
    },
    "ollama": {
        "models": [
            {
                "api": "deepseek-r1:1.5b",
                "name": "DeepSeek-R1 (1.5B)",
                "family": "deepseek",
                "group": "deepseek-r1",
            },
            {
                "api": "deepseek-r1:7b",
                "name": "DeepSeek-R1 (7B)",
                "family": "deepseek",
                "group": "deepseek-r1",
            },
            {
                "api": "deepseek-r1:8b",
                "name": "DeepSeek-R1 (8B)",
                "family": "deepseek",
                "group": "deepseek-r1",
            },
            {
                "api": "deepseek-r1:14b",
                "name": "DeepSeek-R1 (14B)",
                "family": "deepseek",
                "group": "deepseek-r1",
            },
            {
                "api": "deepseek-r1:32b",
                "name": "DeepSeek-R1 (32B)",
                "family": "deepseek",
                "group": "deepseek-r1",
            },
            {
                "api": "deepseek-r1:70b",
                "name": "DeepSeek-R1 (70B)",
                "family": "deepseek",
                "group": "deepseek-r1",
            },
            {
                "api": "deepseek-r1:671b",
                "name": "DeepSeek-R1 (671B)",
                "family": "deepseek",
                "group": "deepseek-r1",
            },
            {
                "api": "gemma2:2b",
                "name": "Gemma 2 (2B)",
                "family": "gemma",
                "group": "gemma2",
            },
            {
                "api": "gemma2:9b",
                "name": "Gemma 2 (9B)",
                "family": "gemma",
                "group": "gemma2",
            },
            {
                "api": "gemma2:27b",
                "name": "Gemma 2 (27B)",
                "family": "gemma",
                "group": "gemma2",
            },
            {
                "api": "gemma:2b",
                "name": "Gemma (Gemma 1.1) (2B)",
                "family": "gemma",
                "group": "gemma1",
            },
            {
                "api": "gemma:7b",
                "name": "Gemma (Gemma 1.1) (7B)",
                "family": "gemma",
                "group": "gemma1",
            },
            {
                "api": "llama3.3:70b",
                "name": "Llama 3.3 (70B)",
                "family": "llama",
                "group": "llama3.3",
            },
            {
                "api": "smollm:135m",
                "name": "SmolLM (135M)",
                "family": "smollm",
                "group": "smollm",
            },
            {
                "api": "smollm:360m",
                "name": "SmolLM (360M)",
                "family": "smollm",
                "group": "smollm",
            },
            {
                "api": "smollm:1.7b",
                "name": "SmolLM (1.7B)",
                "family": "smollm",
                "group": "smollm",
            },
            {
                "api": "dolphin-llama3:8b",
                "name": "Dolphin-Llama3 (8B)",
                "family": "dolphin",
                "group": "dolphin-llama3",
            },
            {
                "api": "dolphin-llama3:70b",
                "name": "Dolphin-Llama3 (70B)",
                "family": "dolphin",
                "group": "dolphin-llama3",
            },
            {
                "api": "hermes3:3b",
                "name": "Hermes 3 (3B)",
                "family": "hermes",
                "group": "hermes3",
            },
            {
                "api": "hermes3:8b",
                "name": "Hermes 3 (8B)",
                "family": "hermes",
                "group": "hermes3",
            },
            {
                "api": "hermes3:70b",
                "name": "Hermes 3 (70B)",
                "family": "hermes",
                "group": "hermes3",
            },
            {
                "api": "hermes3:405b",
                "name": "Hermes 3 (405B)",
                "family": "hermes",
                "group": "hermes3",
            },
            {
                "api": "falcon3:1b",
                "name": "Falcon3 (1B)",
                "family": "falcon3",
                "group": "falcon3",
            },
            {
                "api": "falcon3:3b",
                "name": "Falcon3 (3B)",
                "family": "falcon3",
                "group": "falcon3",
            },
            {
                "api": "falcon3:7b",
                "name": "Falcon3 (7B)",
                "family": "falcon3",
                "group": "falcon3",
            },
            {
                "api": "falcon3:10b",
                "name": "Falcon3 (10B)",
                "family": "falcon3",
                "group": "falcon3",
            },
            {
                "api": "qwen3-coder:30b",
                "name": "Qwen3-Coder (30B)",
                "family": "qwen",
                "group": "qwen3-coder",
            },
            {
                "api": "qwen3-coder:480b",
                "name": "Qwen3-Coder (480B)",
                "family": "qwen",
                "group": "qwen3-coder",
            },
        ]
    },
}



import os
import json

CONFIG_PATH = os.path.expanduser("~/.config/aegish.json")


def load_config():
    if os.path.exists(CONFIG_PATH):
        with open(CONFIG_PATH, 'r') as f:
            return json.load(f)
    print("Please run 'ag --config'")
    return {}

def save_config(config):
    with open(CONFIG_PATH, 'w') as f:
        json.dump(config, f, indent=2)


def configure_interactive():
    from aegish.models import PROVIDERS
    config = load_config()
    BOLD = '\033[1m'
    CYAN = '\033[36m'
    GREEN = '\033[32m'
    RESET = '\033[0m'
    YELLOW = '\033[33m'

    print(f"\n{BOLD}{CYAN}=== aegish Configuration ==={RESET}")
    print(f"{YELLOW}Select your provider, model, and API key if needed.{RESET}\n")

    # Provider selection
    provider_list = list(PROVIDERS.keys())
    for idx, name in enumerate(provider_list, 1):
        print(f"{idx}: {name.title()}")
    while True:
        try:
            provider_choice = int((input(f"{BOLD}Provider (number){RESET}: ") or '').strip())
            if 1 <= provider_choice <= len(provider_list):
                provider = provider_list[provider_choice - 1]
                break
        except ValueError:
            pass
        print(f"{YELLOW}Invalid choice. Please enter a number from the list.{RESET}")

    # Model selection (show available models)
    models = PROVIDERS[provider]["models"]
    print(f"{BOLD}Available models for {provider.title()}:{RESET}")
    for idx, m in enumerate(models, 1):
        print(f"  {idx}: {m['name']}")
    while True:
        try:
            model_choice = int((input(f"{BOLD}Model (number){RESET}: ") or '').strip())
            if 1 <= model_choice <= len(models):
                model = models[model_choice - 1]
                break
        except ValueError:
            pass
        print(f"{YELLOW}Invalid choice. Please enter a number from the list.{RESET}")

    # API Key (only for cloud providers)
    api_key = ''
    if provider in ('gemini', 'claude', 'openai'):
        api_key = (input(f"{BOLD}API Key for {provider.title()}{RESET}: ") or '').strip()

    config['provider'] = provider
    config['model'] = model
    if api_key:
        config['api_key'] = api_key
    else:
        config.pop('api_key', None)
    save_config(config)
    print(f"\n{GREEN}✔ Configuration saved successfully!{RESET}\n")


def configure_system_prompt():
    BOLD = '\033[1m'
    CYAN = '\033[36m'
    GREEN = '\033[32m'
    RESET = '\033[0m'
    YELLOW = '\033[33m'

    config = load_config()
    print(f"\n{BOLD}{CYAN}=== System Prompt Configuration ==={RESET}")
    print(f"{YELLOW}What do you want to do? Press the corresponding number{RESET}")
    print(f"{BOLD}1.{RESET} Change System Prompt")
    print(f"{BOLD}2.{RESET} Reset System Prompt to default")

    while True:
        
        choice = (input(f"{BOLD}Choice (1/2){RESET}: ") or '').strip()
        if choice in ('1', '2'):
            break
        print(f"{YELLOW}Invalid choice. Please enter 1 or 2.{RESET}")

    if choice == '1':
        # change the system prompt in the json
        prompt = input(f"{BOLD}Enter the new system prompt:{RESET} ")
        config['system_prompt'] = prompt
        save_config(config)
        print(f"{GREEN}✔ System prompt updated successfully.{RESET}")
    else:
        # remove the system prompt from the config file to reset to default
        config.pop('system_prompt', None)
        save_config(config)
        print(f"{GREEN}✔ System prompt reset to default.{RESET}")



from openai import OpenAI
from aegish.config import load_config


class OpenAIHandler:
    def __init__(self):
        config = load_config()
        self.model = config["model"]["api"]
        self.api_key = config.get('api_key')
        if not self.api_key:
            raise ValueError("No OpenAI API key found in config.")
        self.client = OpenAI(api_key=self.api_key)

    def generate(self, prompt: str) -> str:
        response = self.client.chat.completions.create(
            model=self.model,
            messages=[{"role": "user", "content": prompt}],
            max_tokens=1000
        )
        # OpenAI API returns choices[0].message.content
        return response.choices[0].message.content.strip()


import sys

from aegish.inputhandler import InputHandler
from aegish.config import load_config
from aegish.debug import print_debug



def main():
	# input
	args, user_prompt = InputHandler.get()

	config = load_config()
	provider = config.get('provider')
	
	# system prompt
	SYSTEM_PROMPT = "You are an expert Linux assistant. Convert the user's natural language request into a safe, correct Linux shell command. If the request is ambiguous or incomplete, infer the most helpful and complete command. For example, if the user says 'git commit', output 'git add . && git commit' to ensure all changes are included. Only output the command, and combine steps when it improves usability."

	# if the config contains a custom system prompt use that else use this
	if 'system_prompt' in config:
		SYSTEM_PROMPT = config.get('system_prompt')

	prompt = f"{SYSTEM_PROMPT}\nUser: {user_prompt}"

	try:
		# Handle the ai providers
		handler = None
		response = None

		if(provider == 'gemini'):
			from aegish.gemini import GeminiHandler
			handler = GeminiHandler()
		elif(provider == 'ollama'):
			from aegish.ollama import OllamaHandler
			handler = OllamaHandler()
		elif(provider == 'openai'):
			from aegish.chatgpt import OpenAIHandler
			handler = OpenAIHandler()
		elif(provider == 'claude'):
			from aegish.claude import ClaudeHandler
			handler = ClaudeHandler()
		else:
			raise ValueError(f"Unknown provider: {provider}")

		response = handler.generate(prompt)
		print_debug(f"Raw response from {provider}: {response}")

		from aegish.postprocess import PostProcessor
		command = PostProcessor.clean(response)

	except Exception as e:
		print(f"Provider error: {e}", file=sys.stderr)
		print(f"Have you run ag --config and saved your preferences? Aegish won't work until you have done so.")
		sys.exit(3)

	# Safety check
	if not args.no_safety:
		from aegish.safetychecker import SafetyChecker
		if not SafetyChecker.confirm_and_run(command):
			sys.exit(0)

	# Prinot only should just only print
	if args.print_only:
		print(command)
		return

	# Run the command
	from aegish.commandrunner import CommandRunner 
	CommandRunner.run(command)



if __name__ == '__main__':
	main()



from aegish.gemini import GeminiHandler
import sys
import os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '../src')))

def test_gemini_handler():
    handler = GeminiHandler()
    # if handler returns anything at all pass the test
    assert handler is not None
    response = handler.generate("echo Hello World")
    assert response is not None




import argparse
import sys
from aegish.debug import set_debug


class InputHandler:

    @staticmethod
    def get():
        parser = argparse.ArgumentParser(description="Converting natural language to linux prompts")
        parser.add_argument('text', nargs=argparse.REMAINDER, help='The natural language command to convert.')
        parser.add_argument('-p', '--print-only', action='store_true', help='Only print the generated command, do not run it')
        parser.add_argument('-n', '--no-safety', action='store_true', help='Disable safety checks on generated commands')
        parser.add_argument('--configure', '--config', action='store_true', dest='configure', help='Configure provider and models')
        parser.add_argument('--configure_system_prompt', '--sys-prompt', action='store_true', dest='configure_system_prompt', help='Configure the system prompt')
        parser.add_argument('-d', '--debug', '--verbose', '-v', action='store_true', help='Enable verbose/debug output')
        args = parser.parse_args()

        # Enter configure mode if --configure
        if args.configure:
            from aegish.config import configure_interactive, configure_system_prompt
            try: 
                configure_interactive()
            except KeyboardInterrupt:
                print("\nquit")
                sys.exit(0)
            sys.exit(0)

        if args.configure_system_prompt:
            try:
                configure_system_prompt()
            except KeyboardInterrupt:
                print("\nquit")
                sys.exit(0)
        # Print debug messages if debug is selected
        set_debug(args.debug)

        # If the text is blank, with no text
        if not args.text:
            print("No input provided. Usage: ag <command> \n Use ag --help to view flags. \n Make sure you run ag --config before first usage", file=sys.stderr)
            sys.exit(1)

        user_prompt = ' '.join(args.text).strip() if args.text else sys.stdin.read().strip()
        return args, user_prompt
